@@ -24,20 +24,14 @@ isAsyncGenerator=lambda x: "async_generator" in str(type(x)) #TODO: howto better
 
 value=lambda x: x.get() if isinstance(x,ReactiveProp) else x
 
-def convjs(l:list)->list:
-    ll=[]
-    for i in l:
-        if i is None:
-            ll.append("null")
-        elif isinstance(i,bool):
-            ll.append( i and "true" or "false")
-        elif isinstance(i,bytes):   # bind b"" -> js
-            ll.append( i.decode() )
-        elif isinstance(i,str):
-            ll.append( "'%s'" % html.escape(i))
+def jjs(obj): #TODO: not optimal ... do better than pattern subst ;-)
+    """ json dumps (js is b'' (bytes)) """
+    def my(obj):
+        if isinstance(obj,bytes):
+            return "<:<:%s:>:>" % obj.decode()
         else:
-            ll.append( str(i) )
-    return ll
+            return guy.serialize(obj)
+    return guy.json.dumps(obj, default=my).replace('"<:<:',"").replace(':>:>"',"")
 
 def log(*a):
     #~ print(*a)
@@ -223,15 +217,19 @@ class GTag:
             del k["parent"]
         else:
             # guess parent
-            frame = sys._getframe(1)
-            arguments = frame.f_code.co_argcount
-            if arguments == 0:
-                parent=None
-            else:
-                caller_calls_self = frame.f_code.co_varnames[0]
-                parent=frame.f_locals[caller_calls_self]
-                assert isinstance(parent,GTag)
-                if parent is self: parent=None  #for TU only
+            lvl=0
+            while True:
+                lvl+=1
+                frame = sys._getframe(lvl)
+                arguments = frame.f_code.co_argcount
+                if arguments == 0:
+                    parent=None
+                    break
+                else:
+                    caller_calls_self = frame.f_code.co_varnames[0]
+                    parent=frame.f_locals[caller_calls_self]
+                    if isinstance(parent,GTag):
+                        break
 
         self._id="%s_%s" % (self.__class__.__name__,hex(id(self))[2:])
         self._childs=[]     #<- this is cleared at each rendering
@@ -241,6 +239,7 @@ class GTag:
 
         log("INIT",repr(self))
         self.init(*self._args,**self._kargs)
+        self._childs=[]     #<- clear innerchilds (creating during child phase), to avoid to appears in child
 
         self._tag = self.build()
 
@@ -255,8 +254,7 @@ class GTag:
                 inners=_gc(obj,lvl+1)
                 ll.extend( [i+' (INNER)' for i in inners] )
             for obj in g._childs:
-                if obj not in g.innerChilds:
-                    ll.extend( _gc(obj,lvl+1) )
+                ll.extend( _gc(obj,lvl+1) )
             return ll
         return "\n".join(_gc(self,0))
 
@@ -272,8 +270,7 @@ class GTag:
             for obj in g.innerChilds:
                 d.update( _gc(obj) )
             for obj in g._childs:
-                if obj not in g.innerChilds:
-                    d.update( _gc(obj) )
+                d.update( _gc(obj) )
             return d
 
         return _gc(self)
@@ -321,11 +318,11 @@ class GTag:
                     else:
                         return ReactiveProp(self.__dict__,name)
                 elif name in dir(self):   # bind a self.method    -> return a js/string for a guy's call in js side
-                    def _(*args):
-                        if args:
-                            return "self.bindUpdate('%s',GID,'%s',%s)" % (self.id,name,",".join(convjs(list(args))) ) #TODO: escaping here ! (and the render/str ?) json here !
+                    def _(*args,**kargs):
+                        if args or kargs:
+                            return "self.bindUpdate('%s',GID,'%s',%s,%s)" % (self.id,name,jjs(args),jjs(kargs))
                         else:
-                            return "self.bindUpdate('%s',GID,'%s')" % (self.id,name)
+                            return "self.bindUpdate('%s',GID,'%s',[],{})" % (self.id,name)
                     return _
                 else:
                     raise Exception("Unknown method/attribut '%s' in '%s'"%(name,self.__class__.__name__))
@@ -335,7 +332,7 @@ class GTag:
     def _clone(self): #TODO: not clear here ... need redone
         assert self._parent==None,"Can't clone a gtag which is not the main"
         props={k:v for k,v in self.__dict__.items() if k[0]!="_" or k=="_call"}
-        gtag = self.__class__(*self._args,**self._kargs,parent=None)
+        gtag = self.__class__(*self._args,**self._kargs,parent=None) # parent=None, will avoid guess parent ! (it makes sense, because you can clone only mains)
         gtag.__dict__.update(props)
         gtag.init(*self._args,**self._kargs)
         gtag._rebuild()
@@ -384,17 +381,24 @@ class GTag:
         self._tag=self.build()
 
     def __str__(self):
+
+        def getTagIded(o):
+            if isinstance(o,Tag):
+                o.id=self.id
+            elif isinstance(o,GTag):
+                o=getTagIded(o._tag)
+                o.id=self.id
+            else:
+                o=Tag.span(o)
+                o.id=self.id
+            return o
+
         log("___rendering",repr(self))
         o= self._tag
         if o is None:
             return ""
         else:
-            if isinstance(o,Tag):
-                o.id=self.id
-            elif isinstance(o,GTag):
-                assert isinstance(o._tag,Tag)
-                o=o._tag        #TODO: recursivity here ?!... wtf if gtag return a gtag
-                o.id=self.id
+            o=getTagIded(o)
             return str(o)
 
     def __repr__(self):
@@ -533,7 +537,7 @@ class GTagApp(guy.Guy):
         log(">>>Force UPDATE:",repr(g))
         await self.js.forceUpdateManually(js)
 
-    async def bindUpdate(self,id:str,gid:str,method:str,*args):
+    async def bindUpdate(self,id:str,gid:str,method:str,args,kargs):
         """ inner (js exposed) guy method, called by gtag.bind.<method>(*args) """
         if self._ses is None:
             gtag=self._originalGTag
@@ -543,14 +547,14 @@ class GTagApp(guy.Guy):
         #////////////////////////////////////////////////////////////////// THE MAGIC TODO: move to gtag
         obj=gtag._getRef(id)
 
-        log("BINDUPDATE on",repr(gtag),"----->",repr(obj),"%s(%s)"% (method,args))
+        log("BINDUPDATE on",repr(gtag),"----->",repr(obj),"%s(%s %s)"% (method,args,kargs))
         proc=getattr(obj,method)
         toRender=gtag if not Capacity(proc).hasLocal else obj
 
         if asyncio.iscoroutinefunction( proc ):
-            rep=await proc(*args)
+            rep=await proc(*args,**kargs)
         else:
-            rep=proc(*args)
+            rep=proc(*args,**kargs)
 
         if rep:
             if isAsyncGenerator(rep):
